"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const _1 = __importDefault(require("."));
const cli_progress_1 = __importDefault(require("cli-progress"));
const cli_ux_1 = require("cli-ux");
const chalk_1 = __importDefault(require("chalk"));
const cac_1 = require("cac");
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const search_query_parser_1 = __importDefault(require("search-query-parser"));
const config_1 = __importDefault(require("./config"));
let g = new _1.default({
    email: config_1.default.get('auth.email'),
    token: config_1.default.get('auth.token'),
});
exports.cli = cac_1.cac('guru-cli');
function checkAuth() {
    if (!config_1.default.has('auth.email') || !config_1.default.has('auth.token')) {
        console.log('Please login first using "guru-cli login"');
        process.exit();
    }
}
async function backupCards(directoryPath) {
    const filePath = path_1.default.join(path_1.default.resolve(directoryPath), `/guru-cards-backup_${new Date().toJSON()}.json`);
    const cards = await g.getAllCardsRaw();
    await promises_1.default.writeFile(filePath, JSON.stringify(cards, null, 4), 'utf8');
    console.log(`Successfully wrote backup to ${filePath}`);
}
function handleCardsFilter(filter, cardsRaw, cliOptions, options) {
    const keywords = ['collection', 'boards', 'title', 'verifier', 'content'];
    let cards = cardsRaw.map(g._convertCardModel);
    if (!filter) {
        if (options === null || options === void 0 ? void 0 : options.raw)
            return cardsRaw;
        return cards;
    }
    const parsed = search_query_parser_1.default.parse(filter, {
        keywords,
        alwaysArray: true,
        tokenize: true,
        offsets: false,
    });
    if (Object.keys(parsed).includes('boards') && !Object.keys(parsed).includes('collection')) {
        console.log('Please add "collection:example" to filter when filtering for boards');
        process.exit();
    }
    let reFlags = '';
    if (cliOptions.ignoreCase)
        reFlags += 'i';
    for (const keyword of keywords) {
        if (parsed[keyword]) {
            for (const value of parsed[keyword]) {
                const matcher = new RegExp(value, reFlags);
                cards = cards.filter((card) => matcher.test(card[keyword]));
            }
        }
    }
    if (options === null || options === void 0 ? void 0 : options.raw) {
        const rawCardsFiltered = cards.map((card) => cardsRaw.find((cardRaw) => cardRaw.id === card.id));
        return rawCardsFiltered;
    }
    return cards;
}
exports.cli.command('login', 'login and save credentials for guru cli').action(async () => {
    config_1.default.clear();
    console.log('Login to Guru API\n------------------------');
    const email = await cli_ux_1.cli.prompt('Guru Admin Email');
    const token = await cli_ux_1.cli.prompt('Admin API Token');
    g = new _1.default({
        email,
        token,
    });
    const checkAuth = await g.authenticated();
    if (!checkAuth.authenticated) {
        console.log(checkAuth.message);
        return;
    }
    config_1.default.set({
        auth: {
            email,
            token,
        },
    });
    console.log('✅ Successfully authenticated to Guru API. You can now run guru-cli commands.');
});
exports.cli
    .command('find <filter>', 'find cards based on search filter (full ECMAScript compliant regex supported)')
    .example(`Find all cards where content has 'delete me'
guru-cli find 'content:"delete me"'
`)
    .example(`Find all cards where category has "Software" OR "Engineering" and title has "Overview"
guru-cli find 'category:Software,Engineering title:Overview'
`)
    .example(`Find cards where verifier is EXACTLY "bob@example.com"
guru-cli find 'verifier:"^bob@example\\\\.com$"'
`)
    .option('-i, --ignore-case', 'Ignore case in the finding regex string')
    .action(async (filter, options) => {
    checkAuth();
    cli_ux_1.cli.action.start(`Searching for cards`);
    const cardsRaw = await g.getAllCardsRaw();
    const filteredCards = handleCardsFilter(filter, cardsRaw, options);
    if (!filteredCards.length) {
        cli_ux_1.cli.action.stop(`❌ ${filteredCards.length} cards found matching your filter`);
        return;
    }
    cli_ux_1.cli.action.stop(`✅  ${filteredCards.length} cards matching your filter\n---------------------------`);
    filteredCards.map((card) => console.log(`${chalk_1.default.bold.blueBright(card.title)} (${card.link})`));
});
exports.cli
    .command('find-replace <find> <replace> [filter]', 'Find and replace across guru articles (supports regex in find) with optional filtering')
    .option('-f, --force', 'Skip confirmation, BE CAREFUL WITH THIS OPTION!')
    .option('-i, --ignore-case', 'Ignore case in the finding regex string')
    .option('-C, --confirm', 'confirm making the changes (required to push modifications to guru)')
    .option('-w, --preview-width <size>', 'characters of text around the match when in preview mode (no --confirm)')
    .action(async (find, replace, filter, options) => {
    checkAuth();
    cli_ux_1.cli.action.start('Finding cards based on filter');
    const cardsRaw = await g.getAllCardsRaw();
    const filteredCards = handleCardsFilter(filter, cardsRaw, options, { raw: true });
    let reFlags = 'g';
    if (options.ignoreCase)
        reFlags += 'i';
    const matchedCards = filteredCards.filter((card) => new RegExp(find, reFlags).test(card.content));
    if (!matchedCards.length) {
        cli_ux_1.cli.action.stop(`❌ ${matchedCards.length} cards found matching "${find}"`);
        return;
    }
    cli_ux_1.cli.action.stop(`✅ ${matchedCards.length} cards found matching "${find}"`);
    if (!options.confirm) {
        console.log('Preview:\n------------------------');
        for (const card of matchedCards) {
            console.log(`${chalk_1.default.bold.blueBright(card.preferredPhrase)} (${chalk_1.default.blue(`https://app.getguru.com/card/${card.slug}`)})`);
            const matches = [...card.content.matchAll(new RegExp(find, reFlags))];
            if (!matches.length)
                throw new Error(`Unknown regex matching error occurred on card ${card.preferredPhrase}`);
            for (const match of matches) {
                const matchText = match[0];
                const matchIndex = match.index;
                const SIZE_AROUND = options.previewWidth ? Number(options.previewWidth) : 40;
                const beforeText = card.content.substring(matchIndex > SIZE_AROUND ? matchIndex - SIZE_AROUND : 0, matchIndex);
                const afterText = card.content.substring(matchIndex + matchText.length, matchIndex + matchText.length + SIZE_AROUND >= card.content.length - 1
                    ? card.content.length - 1
                    : matchIndex + matchText.length + SIZE_AROUND);
                console.log(`${beforeText}${chalk_1.default.bold.redBright(matchText)}${chalk_1.default.bold.green(replace)}${afterText}`);
            }
            console.log('');
        }
        console.log(`Rerun this same command with --confirm to push these changes to these ${matchedCards.length} Guru cards`);
        return;
    }
    if (!options.force) {
        const backup = await cli_ux_1.cli.confirm(`Would you like to backup card data before making changes? (RECOMMENDED) (Y/n)`);
        if (backup) {
            await backupCards('./');
        }
        const confirm = await cli_ux_1.cli.confirm(`This will replace all instances matching "${chalk_1.default.bold.redBright(find)}" with ${chalk_1.default.bold.green(replace)} in ${matchedCards.length} cards. Continue? (Y/n)`);
        if (!confirm)
            return;
    }
    for (const card of matchedCards) {
        card.content = card.content.replace(new RegExp(find, reFlags), replace);
        await g.updateCardRaw(card);
    }
    console.log(`Successfully replaced "${chalk_1.default.bold.redBright(find)}" with "${chalk_1.default.bold.green(replace)}" across ${matchedCards.length} cards.`);
});
exports.cli
    .command('verify-expired [filter]', 'Verifies all expired cards with optional filtering (see README)')
    .option('-i, --ignore-case', 'Ignore case in the finding regex string')
    .option('-f, --force', 'Skip confirmation, BE CAREFUL WITH THIS OPTION!')
    .action(async (filter, options) => {
    checkAuth();
    cli_ux_1.cli.action.start('Gathering all expired cards');
    const expiredCards = await g.getAllExpiredCardsRaw();
    const filteredCards = handleCardsFilter(filter, expiredCards, options);
    cli_ux_1.cli.action.stop(`✅ ${filteredCards.length} expired cards found.`);
    if (!filteredCards.length) {
        console.log('You have no expired cards currently!');
        return;
    }
    console.log('Preview:\n------------------------');
    for (const card of filteredCards) {
        console.log(chalk_1.default.bold.blueBright(card.title));
    }
    if (!options.force) {
        const confirm = await cli_ux_1.cli.confirm(`This will verify ${filteredCards.length} cards currently in expired state. Continue? (Y/n)`);
        if (!confirm)
            return;
    }
    const progress = new cli_progress_1.default.Bar({
        format: `Verifying Expired Cards ${chalk_1.default.hex('#7F4BAE')('{bar}')} {percentage}% | Current Card: {value}/{total} | Name: ${chalk_1.default.bold.hex('#D7E150')('{currentCard}')}`,
    }, cli_progress_1.default.Presets.shades_grey);
    progress.start(filteredCards.length, 0, { currentCard: '' });
    for (const card of filteredCards) {
        progress.increment(1, { currentCard: card.title });
        await g.verifyCardsByTitle(card.title);
    }
    progress.stop();
});
exports.cli
    .command('backup-cards <directory-path>', 'create a backup of all cards in Guru account in JSON at specified file path')
    .action(async (directoryPath) => {
    checkAuth();
    await backupCards(directoryPath);
});
exports.cli.command('[...default]', 'display help if no command given').usage('--help').action(exports.cli.outputHelp);
exports.cli.help();
if (require.main === module)
    exports.cli.parse();
