"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Guru = void 0;
const axios_1 = __importDefault(require("axios"));
class Guru {
    constructor(_config) {
        this.email = _config.email;
        this.token = _config.token;
        this.httpConfig = {
            baseURL: 'https://api.getguru.com/api/v1',
            auth: {
                username: this.email,
                password: this.token,
            },
        };
    }
    async _request(method, url, data, axiosOptions) {
        try {
            const res = await axios_1.default(url, {
                method,
                data,
                ...this.httpConfig,
                ...(axiosOptions !== null && axiosOptions !== void 0 ? axiosOptions : {}),
            });
            return res;
        }
        catch (err) {
            if (err.response.status === 401) {
                throw new Error('Could not authenticate to Guru, check credentials and try again.');
            }
            throw err;
        }
    }
    _convertCardModel(cardRaw) {
        var _a;
        return {
            id: cardRaw.id,
            title: cardRaw.preferredPhrase,
            owner: cardRaw.owner.email,
            firstName: cardRaw.owner.firstName,
            lastName: cardRaw.owner.lastName,
            verifier: (_a = cardRaw.verifiers[0]) === null || _a === void 0 ? void 0 : _a.user.email,
            collection: cardRaw.collection.name,
            boards: cardRaw.boards ? cardRaw.boards.map((board) => board.title) : [],
            content: cardRaw.content,
            verificationDate: cardRaw.nextVerificationDate,
            verificationState: cardRaw.verificationState,
            verificationInterval: cardRaw.verificationInterval,
            link: `https://app.getguru.com/card/${cardRaw.slug}`,
        };
    }
    async authenticated() {
        try {
            await this._request('GET', '/tasks/dashboard');
            return {
                authenticated: true,
                message: '',
            };
        }
        catch (err) {
            return {
                authenticated: false,
                message: err.message,
            };
        }
    }
    async getAllCardsRaw() {
        const allCards = [];
        let res = await this._request('POST', '/search/cardmgr', {});
        allCards.push(...res.data);
        while (res.headers.link) {
            const pageLink = res.headers.link.match(/(?<=<)(.*)(?=>)/gs)[0];
            res = await this._request('POST', pageLink, {});
            allCards.push(...res.data);
        }
        return allCards;
    }
    async getAllCards() {
        const allCards = await this.getAllCardsRaw();
        return allCards.map(this._convertCardModel);
    }
    async getCardsByTitle(cardTitle) {
        const { data: res } = await this._request('POST', `/search/cardmgr`, {
            queryType: null,
            sorts: null,
            query: { nestedExpressions: [{ type: 'title', value: cardTitle, op: 'IS' }], op: 'AND', type: 'grouping' },
            collectionIds: [],
        });
        if (!res.length)
            throw new Error(`Unable to find card with title: ${cardTitle}`);
        const cards = res;
        return cards.map(this._convertCardModel);
    }
    async verifyCardsByTitle(cardTitle) {
        let cards;
        try {
            cards = await this.getCardsByTitle(cardTitle);
        }
        catch {
            return {
                status: 'ERROR',
                message: `Could not find card with title of ${cardTitle}`,
            };
        }
        for (const card of cards) {
            const impersonationToken = await this.createImpersonationToken(card.verifier);
            await this._request('POST', '/cards/bulkop', {
                action: { type: 'verify-card' },
                items: { type: 'id', cardIds: [card.id] },
            }, impersonationToken
                ? {
                    auth: {
                        username: card.verifier,
                        password: impersonationToken,
                    },
                }
                : undefined);
        }
        return {
            status: 'OK',
            message: `Success verifying "${cards.length}" card(s) with title of ${cardTitle}`,
        };
    }
    async getAllExpiredCardsRaw() {
        const expiredCards = [];
        const cards = await this.getAllCardsRaw();
        for (const card of cards) {
            if (card.verificationState === 'NEEDS_VERIFICATION') {
                const { data: cardDetail } = await this._request('GET', `/cards/${card.id}`);
                if (!cardDetail.verificationReasons.length || cardDetail.verificationReasons[0] === 'EXPIRED') {
                    expiredCards.push(card);
                }
            }
        }
        return expiredCards;
    }
    async updateCardRaw(cardRaw) {
        var _a;
        const verifierEmail = (_a = cardRaw.verifiers[0]) === null || _a === void 0 ? void 0 : _a.user.email;
        if (!verifierEmail)
            throw new Error('Unable to find verifier email in card data');
        const { data: res } = await this._request('PUT', `/cards/${cardRaw.id}`, cardRaw, {
            auth: {
                username: verifierEmail,
                password: await this.createImpersonationToken(verifierEmail),
            },
        });
        return res;
    }
    async getAllCollections() {
        const { data: res } = await this._request('GET', `/collections`);
        return res;
    }
    async createImpersonationToken(userEmail) {
        if (userEmail === this.email)
            return this.token;
        const res = await axios_1.default({
            method: 'POST',
            url: `https://api.getguru.com/api/v1/api-tokens/users/${userEmail}`,
            data: {},
            auth: {
                username: this.email,
                password: this.token,
            },
        });
        return res.data.token;
    }
}
exports.Guru = Guru;
